<?php
/**
 * @Copyright
 * @package        EasyCalcCheck Captcha - ECCC for Joomla! 3
 * @author         Viktor Vogel <admin@kubik-rubik.de>
 * @version        3.1.0 - 2018-07-17
 * @link           https://joomla-extensions.kubik-rubik.de/ecc-easycalccheck-plus
 *
 * @license        GNU/GPL
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
defined('_JEXEC') || die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\User\UserHelper;

class PlgCaptchaEasyCalcCheckCaptcha extends CMSPlugin
{
    /**
     * Loads the language file automatically in the frontend
     *
     * @var bool
     */
    protected $autoloadLanguage = true;

    /**
     * Current session object
     *
     * @var \Joomla\CMS\Session\Session
     */
    protected $session;

    /**
     * Maximum value for the summands of the maths exercise
     *
     * @var int
     */
    protected $maxValue;

    /**
     * Use a hidden field in the spam check
     *
     * @var int
     */
    protected $hiddenField;

    /**
     * Use a hidden field in the spam check
     *
     * @var int
     */
    protected $selfDefinedQuestion;

    /**
     * PlgCaptchaEasyCalcCheckCaptcha constructor.
     *
     * @param $subject
     * @param $config
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

        $this->session = Factory::getSession();
        $this->maxValue = (int) $this->params->get('maxValue', 20);
        $this->hiddenField = (bool) $this->params->get('hiddenField', true);
        $this->selfDefinedQuestion = (bool) $this->validateSelfDefinedQuestion();
    }

    /**
     * Validates whether the self-defined question can be used
     *
     * @return bool
     */
    private function validateSelfDefinedQuestion()
    {
        return $this->params->get('selfDefinedQuestion') && $this->params->get('selfDefinedQuestionQuestion') && $this->params->get('selfDefinedQuestionAnswer');
    }

    /**
     * Initialises the spam check protection
     *
     * @param string $id
     *
     * @throws Exception
     */
    public function onInit($id = 'eccl')
    {
        $this->session->set('initTimeStamp', time(), 'eccl');

        if (!$this->selfDefinedQuestion) {
            $this->session->set('spamCheckNumber1', mt_rand(0, $this->maxValue), 'eccl');
            $this->session->set('spamCheckNumber2', mt_rand(0, $this->maxValue), 'eccl');
        }

        if ($this->hiddenField) {
            $hiddenId = $this->createRandomString();
            Factory::getDocument()->addStyleDeclaration('#' . $hiddenId . ' {display: none;}');
            $this->session->set('hiddenId', $hiddenId, 'eccl');
        }
    }

    /**
     * Creates a random string with an alphabetic character in the beginning (required for a valid CSS name)
     *
     * @return string
     */
    private function createRandomString()
    {
        $charactersArray = array_merge(range('a', 'z'), range('A', 'Z'));
        $randomString = $charactersArray[mt_rand(0, count($charactersArray) - 1)];
        $randomString .= UserHelper::genRandomPassword(mt_rand(6, 12));

        return $randomString;
    }

    /**
     * Gets the challenge HTML output
     *
     * @param   string $name
     * @param   string $id
     * @param   string $class
     *
     * @return  string  The HTML to be embedded in the form.
     */
    public function onDisplay($name = null, $id = 'eccl', $class = '')
    {
        $spamCheckField = $this->createCaptchaOutput($name, $id);
        $spamCheckField .= $this->createHiddenFieldOutput();

        return $spamCheckField;
    }

    private function createCaptchaOutput($name, $id)
    {
        if (!$this->selfDefinedQuestion) {
            $spamCheckNumber1 = $this->session->get('spamCheckNumber1', 0, 'eccl');
            $spamCheckNumber1 = $this->formatNumbersSpellout($spamCheckNumber1);
            $spamCheckNumber2 = $this->session->get('spamCheckNumber2', 0, 'eccl');
            $spamCheckNumber2 = $this->formatNumbersSpellout($spamCheckNumber2);

            return '<input type="text" name="' . $name . '" id="' . $id . '" size="3" class="inputbox validate-numeric required" value="" required="required" placeholder="' . $spamCheckNumber1 . ' + ' . $spamCheckNumber2 . '"/>';
        }

        $selfDefinedQuestionQuestion = $this->params->get('selfDefinedQuestionQuestion', '');

        return '<input type="text" name="' . $name . '" id="' . $id . '" size="3" class="inputbox required" value="" required="required" placeholder="' . $selfDefinedQuestionQuestion . '"/>';
    }

    /**
     * Applies spellout rule-based format if class NumberFormatter exists
     *
     * @param $number
     *
     * @return string
     */
    private function formatNumbersSpellout($number)
    {
        if (class_exists('NumberFormatter')) {
            $formatter = new NumberFormatter(Factory::getLanguage()->getTag(), NumberFormatter::SPELLOUT);

            return $formatter->format($number);
        }

        return $number;
    }

    private function createHiddenFieldOutput()
    {
        $hiddenFieldOutput = '';

        if ($this->hiddenField) {
            $hiddenId = $this->session->get('hiddenId', false, 'eccl');

            $hiddenFieldOutput .= '<input type="text" name="' . $hiddenId . '" id="' . $hiddenId . '" size="3" class="inputbox" value="" placeholder="' . $this->formatNumbersSpellout(mt_rand(0, $this->maxValue)) . ' + ' . $this->formatNumbersSpellout(mt_rand(0, $this->maxValue)) . '"/>';
        }

        return $hiddenFieldOutput;
    }

    /**
     * Validates the response from the user
     *
     * @param null $result
     *
     * @return bool
     * @throws Exception
     */
    public function onCheckAnswer($result = null)
    {
        if (!$this->performChecks($result)) {
            Factory::getApplication()->enqueueMessage(Text::_('PLG_ECCC_YOUHAVENOTRESOLVEDOURSPAMCHECK'), 'error');

            return false;
        }

        return true;
    }

    /**
     * Performs the validation checks
     *
     * @param $result
     *
     * @return bool
     * @throws Exception
     */
    private function performChecks($result)
    {
        if (is_null($result)) {
            return false;
        }

        $timeLockSeconds = (int) $this->params->get('timeLockSeconds', 5);
        $initTimeStamp = $this->session->get('initTimeStamp', null, 'eccl');
        $this->session->clear('initTimeStamp', 'eccl');

        if (is_null($initTimeStamp) || time() - $timeLockSeconds <= (int) $initTimeStamp) {
            return false;
        }

        if ($this->hiddenField) {
            $hiddenId = $this->session->get('hiddenId', false, 'eccl');
            $hiddenFieldValue = Factory::getApplication()->input->get($hiddenId, '', 'RAW');

            if (!empty($hiddenFieldValue)) {
                return false;
            }
        }

        if (!$this->performCaptchaCheck($result)) {
            return false;
        }

        return true;
    }

    /**
     * Performs the captcah validation check
     *
     * @param $result
     *
     * @return bool
     */
    private function performCaptchaCheck($result)
    {
        if (!$this->selfDefinedQuestion) {
            $spamCheckNumber1 = (int) $this->session->get('spamCheckNumber1', null, 'eccl');
            $this->session->clear('spamCheckNumber1', 'eccl');

            $spamCheckNumber2 = (int) $this->session->get('spamCheckNumber2', null, 'eccl');
            $this->session->clear('spamCheckNumber2', 'eccl');

            if (is_null($spamCheckNumber1) || is_null($spamCheckNumber2)) {
                return false;
            }

            if ($spamCheckNumber1 + $spamCheckNumber2 !== (int) $result) {
                return false;
            }
			
			return true;
        }

        $selfDefinedQuestionAnswer = $this->params->get('selfDefinedQuestionAnswer');

        if ($selfDefinedQuestionAnswer !== $result) {
            return false;
        }

        return true;
    }
}
